import os
import logging
import subprocess
import tempfile
import shutil

import ansible_bender
from ansible_bender import callback_plugins
from ansible_bender.exceptions import AbBuildUnsuccesful
from ansible_bender.utils import run_cmd, ap_command_exists


logger = logging.getLogger(__name__)
A_CFG_TEMPLATE = """\
[defaults]
retry_files_enabled = False
# when user is changed, ansible might not be able to write to /.ansible
remote_tmp = /tmp
callback_plugins={0}
callback_whitelist=snapshoter\n
"""


# TODO: implement ansible_args and utilize it in tests to provide mega debug output
def run_playbook(playbook_path, inventory_path, a_cfg_path, connection, extra_variables=None,
                 ansible_args=None, debug=False, environment=None):
    """
    run selected ansible playbook and return output from ansible-playbook run

    :param playbook_path:
    :param inventory_path:
    :param a_cfg_path:
    :param connection:
    :param extra_variables:
    :param ansible_args:
    :param debug:
    :param environment:

    :return: output
    """
    ap = ap_command_exists()
    cmd_args = [
        ap,
        "-i", inventory_path,
        "-c", connection,

    ]
    if debug:
        cmd_args += ["-vvv"]
    if extra_variables:
        cmd_args += ["--extra-vars"] + \
                    [" ".join(
                        ["{}={}".format(k, v)
                         for k, v in extra_variables.items()]
                    )]
    cmd_args += [playbook_path]
    logger.debug("%s", " ".join(cmd_args))

    env = os.environ.copy()
    if environment:
        env.update(environment)
    env["ANSIBLE_CONFIG"] = a_cfg_path

    # TODO: does ansible have an API?
    try:
        return run_cmd(
            cmd_args,
            print_output=True,
            env=env,
            return_all_output=True,
        )
    except subprocess.CalledProcessError as ex:
        raise AbBuildUnsuccesful("ansible-playbook execution failed: %s" % ex, ex.output)


class AnsibleRunner:
    """
    Run ansible on provided artifact using the Builder interface
    """

    def __init__(self, playbook_path, builder, build, debug=False):
        self.build_i = build
        self.pb = playbook_path
        self.builder = builder
        self.debug = debug

    def _create_inventory_file(self, fd, python_interpreter):
        fd.write(
            '%s ansible_connection="%s" ansible_python_interpreter="%s"\n' % (
                self.builder.ansible_host,
                self.builder.ansible_connection,
                python_interpreter
            )
        )

    def _create_ansible_cfg(self, fd):
        callback_plugins_dir = os.path.dirname(callback_plugins.__file__)
        fd.write(A_CFG_TEMPLATE.format(callback_plugins_dir))

    def _get_path_our_site(self):
        """ return a path to a directory which contains ansible_bender installation """
        # pip in Fedora installs to /usr/local which is on default pythonpath but when ansible invokes
        # the callback plugin, that directory is not on sys.path: wat?
        # hence, let's add the site ab is installed in to sys.path
        return os.path.dirname(os.path.dirname(ansible_bender.__file__))

    def build(self, db_path, python_interpreter="/usr/bin/python3"):
        """
        run the playbook against the container

        :return: str, output
        """
        tmp = tempfile.mkdtemp(prefix="ab")

        pythonpath = os.environ.get("PYTHONPATH", "")
        site_path = self._get_path_our_site()
        if pythonpath:
            pythonpath = f"{site_path}:{pythonpath}"
        else:
            pythonpath = site_path

        try:
            environment = {
                "AB_BUILD_ID": self.build_i.build_id,
                "AB_DB_PATH": db_path,
                "PYTHONPATH": pythonpath,  # TODO write an e2e test for this
            }
            inv_path = os.path.join(tmp, "inventory")
            logger.info("creating inventory file %s", inv_path)
            with open(inv_path, "w") as fd:
                self._create_inventory_file(fd, python_interpreter)
            a_cfg_path = os.path.join(tmp, "ansible.cfg")
            with open(a_cfg_path, "w") as fd:
                self._create_ansible_cfg(fd)
            return run_playbook(self.pb, inv_path, a_cfg_path, self.builder.ansible_connection,
                                debug=self.debug, environment=environment)
        finally:
            shutil.rmtree(tmp)



"""
Make sure that we can build using buildah builder.
"""
import json
import os
import re
import subprocess

import pytest

from ansible_bender.builders.buildah_builder import buildah, inspect_buildah_resource, podman_run_cmd
from ..spellbook import basic_playbook_path, base_image, target_image, project_dir, \
    bad_playbook_path, random_word, basic_playbook_path_w_bv


def ab(args, tmpdir_path, debug=False, return_output=False, ignore_result=False):
    """
    python3 -m ab.cli -v build ./playbook.yaml registry.fedoraproject.org/fedora:28 asdqwe-image

    :return:
    """
    cmd = ["python3", "-m", "ansible_bender.cli", "--database-dir", tmpdir_path]
    if debug:
        cmd += ["--debug"]
    if ignore_result:
        return subprocess.call(cmd + args, cwd=project_dir)
    if return_output:
        return subprocess.check_output(cmd + args, cwd=project_dir, universal_newlines=True)
    else:
        # don't use run_cmd here, it makes things complicated
        subprocess.check_call(cmd + args, cwd=project_dir)


def test_output(target_image, tmpdir):
    cmd = ["build", basic_playbook_path, base_image, target_image]
    out = ab(cmd, str(tmpdir), return_output=True, debug=False)
    assert f"Image '{target_image}' was built successfully \\o/" in out
    assert 'Getting image source signatures' in out
    assert not re.match(r'ERROR\s+Getting image source signatures', out)
    assert 'Copying ' in out
    assert not re.match(r'ERROR\s+Copying ', out)
    buildah("inspect", ["-t", "image", target_image])


def test_build_basic_image(target_image, tmpdir):
    cmd = ["build", basic_playbook_path, base_image, target_image]
    ab(cmd, str(tmpdir))

    p_inspect_data = json.loads(subprocess.check_output(["podman", "inspect", "-t", "image", target_image]))[0]
    image_id = p_inspect_data["Id"]

    cmd = ["inspect", "--json"]
    ab_inspect_data = json.loads(ab(cmd, str(tmpdir), return_output=True))
    top_layer_id = ab_inspect_data["layers"][-1]["layer_id"]
    final_image_id = ab_inspect_data["final_layer_id"]

    # these two are really different: top is w/o metadata
    assert image_id != top_layer_id
    assert image_id == final_image_id


def test_build_basic_image_with_env_vars(tmpdir, target_image):
    a_b = "A=B"
    x_y = "X=Y"
    cmd = ["build", "-e", a_b, x_y, "--",
           basic_playbook_path, base_image, target_image]
    ab(cmd, str(tmpdir))
    out = inspect_buildah_resource("image", target_image)
    assert a_b in out["OCIv1"]["config"]["Env"]
    assert x_y in out["OCIv1"]["config"]["Env"]
    e = podman_run_cmd(target_image, ["env"], return_output=True)
    assert a_b in e
    assert x_y in e


def test_build_basic_image_with_wrong_env_vars(tmpdir, target_image):
    la_la_la = "AB-LA-BLA-LA-LA-BLA"
    cmd = ["build", "-e", la_la_la, "--", basic_playbook_path, base_image, target_image]
    with pytest.raises(subprocess.CalledProcessError) as exc:
        ab(cmd, str(tmpdir))
        e = ("There was an error during execution: "
             "Environment variable {} doesn't seem to be "
             "specified in format 'KEY=VALUE'.".format(la_la_la))
        assert e in exc.value.message


def test_build_basic_image_with_labels(tmpdir, target_image):
    a_b = "A=B"
    x_y = "x=y"
    cmd = ["build", "-l", a_b, x_y, "--",
           basic_playbook_path, base_image, target_image]
    ab(cmd, str(tmpdir))
    out = inspect_buildah_resource("image", target_image)
    assert out["OCIv1"]["config"]["Labels"]["A"] == "B"
    assert out["OCIv1"]["config"]["Labels"]["x"] == "y"


def test_build_basic_image_with_build_volumes(tmpdir, target_image):
    real_tmp = str(tmpdir)
    with open(os.path.join(real_tmp, "file.txt"), "w") as fd:
        fd.write("Hello, hello!")
    container_mount = "/asdqwe"
    vol_spec = "%s:%s" % (real_tmp, container_mount)
    cmd = ["build", "--build-volumes", vol_spec, "--",
           basic_playbook_path_w_bv, base_image, target_image]
    ab(cmd, str(tmpdir))

    cmd = ["inspect", "--json"]
    ab_inspect_data = json.loads(ab(cmd, str(tmpdir), return_output=True))
    build_volumes = ab_inspect_data["build_volumes"]

    assert vol_spec in build_volumes


def test_build_basic_image_with_all_params(tmpdir, target_image):
    workdir_path = "/etc"
    l_a_b = "A=B"
    l_x_y = "x=y"
    e_a_b = "A=B"
    e_x_y = "X=Y"
    cmd, cmd_e = "ls -lha", ["ls", "-lha"]
    # FIXME: this doesn't work with user namespaces
    # user = "1000"
    p_80, p_443 = "80", "443"
    runtime_volume = "/var/lib/asdqwe"
    cmd = ["build",
           "-w", workdir_path,
           "-l", l_a_b, l_x_y,
           "-e", e_a_b, e_x_y,
           "--cmd", cmd,
           # "-u", user,
           "-p", p_80, p_443,
           "--runtime-volumes", runtime_volume,
           "--",
           basic_playbook_path, base_image, target_image]
    ab(cmd, str(tmpdir))
    out = inspect_buildah_resource("image", target_image)
    co = out["OCIv1"]["config"]
    assert co["WorkingDir"] == workdir_path
    assert co["Labels"]["A"] == "B"
    assert co["Labels"]["x"] == "y"
    assert e_a_b in co["Env"]
    assert e_x_y in co["Env"]
    assert co["Cmd"] == cmd_e
    # assert co["User"] == user
    assert p_80 in co["ExposedPorts"]
    assert p_443 in co["ExposedPorts"]
    # FIXME: the volume is not set; a bug in buildah or ab?


def test_build_failure(tmpdir):
    target_image = "registry.example.com/ab-test-" + random_word(12) + ":oldest"
    target_failed_image = target_image + "-failed"
    cmd = ["build", bad_playbook_path, base_image, target_image]
    with pytest.raises(subprocess.CalledProcessError):
        ab(cmd, str(tmpdir))
    out = ab(["get-logs"], str(tmpdir), return_output=True)
    assert "PLAY [all]" in out

    p_inspect_data = json.loads(subprocess.check_output(["podman", "inspect", "-t", "image", target_failed_image]))[0]
    image_id = p_inspect_data["Id"]

    cmd = ["inspect", "--json"]
    ab_inspect_data = json.loads(ab(cmd, str(tmpdir), return_output=True))
    top_layer_id = ab_inspect_data["layers"][-1]["layer_id"]
    final_image_id = ab_inspect_data["final_layer_id"]

    assert image_id == top_layer_id
    assert image_id == final_image_id
    assert ab_inspect_data["target_image"] == target_failed_image
    buildah("rmi", [target_failed_image])


def test_two_runs(tmpdir, target_image):
    """ this is a naive test to verify race condition """
    cmd = ["python3", "-m", "ansible_bender.cli", "--database-dir", str(tmpdir),
           "build", basic_playbook_path, base_image,
           target_image]
    p1 = subprocess.Popen(cmd)
    p2 = subprocess.Popen(cmd)
    p1.wait()
    p2.wait()
    assert p1.returncode == 0
    assert p2.returncode == 0


def test_buildah_err_output(tmpdir, capfd):
    cmd = ["build", basic_playbook_path, base_image, "vrerv\\23&^&4//5B/F/BSFD/B"]
    ab(cmd, str(tmpdir), debug=False, ignore_result=True)
    c = capfd.readouterr()
    assert "error parsing target image name" in c.err


def test_push_to_dockerd(target_image, tmpdir):
    cmd = ["build", basic_playbook_path, base_image, target_image]
    ab(cmd, str(tmpdir))
    target = "ab-test-" + random_word(12) + ":oldest"
    cmd = ["push", "docker-daemon:" + target]
    ab(cmd, str(tmpdir))
    try:
        cmd = ["docker", "run", "--rm", target, "cat", "/fun"]
        subprocess.check_call(cmd)
    finally:
        subprocess.check_call(["docker", "rmi", target])




"""
Test Application class
"""
import os
import shutil

from ansible_bender.api import Application
from ansible_bender.builders.base import Build, ImageMetadata, BuildState
from tests.spellbook import dont_cache_playbook_path, change_layering_playbook, data_dir, dont_cache_playbook_path_pre
from ..spellbook import basic_playbook_path, small_basic_playbook_path, base_image, target_image

import pytest
import yaml


@pytest.fixture()
def application(tmpdir):
    database_path = str(tmpdir)
    application = Application(db_path=database_path)  # use debug=True to hunt errors
    yield application
    application.clean()


@pytest.fixture()
def build(target_image):
    build = Build()
    build.playbook_path = basic_playbook_path
    build.base_image = base_image
    build.target_image = target_image
    build.metadata = ImageMetadata()
    build.state = BuildState.NEW
    build.builder_name = "buildah"  # test with all builders
    return build


def test_build_db_metadata(target_image, application, build):
    application.build(build)
    build = application.db.get_build(build.build_id)
    assert build.playbook_path is not None
    assert build.build_finished_time is not None
    assert build.build_start_time is not None
    assert build.log_lines is not None
    logs = "\n".join(build.log_lines)
    assert "PLAY [all]" in logs
    assert "TASK [Gathering Facts]" in logs
    assert "failed=0" in logs


def test_caching(target_image, application, build):
    b2 = Build.from_json(build.to_dict())
    application.build(build)
    b2.build_id = None
    b2.layers = []
    b2.target_image += "2"
    application.build(b2)
    build = application.db.get_build(build.build_id)
    b2 = application.db.get_build(b2.build_id)
    assert [x.layer_id for x in b2.layers[:3]] == [y.layer_id for y in build.layers[:3]]
    assert not b2.layers[4].cached
    assert not build.layers[4].cached
    assert len(build.layers) == 5


def test_disabled_caching(target_image, application, build):
    build.cache_tasks = False
    application.build(build)
    build = application.db.get_build(build.build_id)
    assert len(build.layers) == 5
    assert build.layers[0].cached
    assert not build.layers[1].cached
    assert not build.layers[2].cached
    assert not build.layers[3].cached
    assert not build.layers[4].cached


def test_caching_mechanism(target_image, application, build):
    """ check that previously executed tasks are being loaded from cache and new ones are computed from scratch """
    small_build = Build.from_json(build.to_dict())
    small_build.target_image += "2"
    small_build.playbook_path = small_basic_playbook_path

    application.build(small_build)
    small_build = application.db.get_build(small_build.build_id)
    assert len(small_build.layers) == 2
    assert small_build.layers[0].cached
    assert not small_build.layers[1].cached

    application.build(build)
    build = application.db.get_build(build.build_id)
    assert len(build.layers) == 5
    assert build.layers[0].cached
    assert build.layers[1].cached
    assert not build.layers[2].cached
    assert not build.layers[3].cached
    assert not build.layers[4].cached


def test_no_cache_tag(target_image, application, build):
    """ utilize a playbook which halts caching """
    dont_cache_b = Build.from_json(build.to_dict())
    build.playbook_path = dont_cache_playbook_path_pre

    application.build(build)
    build = application.db.get_build(build.build_id)
    assert len(build.layers) == 4
    assert build.layers[0].cached
    assert not build.layers[1].cached
    assert not build.layers[2].cached
    assert not build.layers[3].cached

    dont_cache_b.target_image += "2"
    dont_cache_b.playbook_path = dont_cache_playbook_path

    application.build(dont_cache_b)
    dont_cache_b = application.db.get_build(dont_cache_b.build_id)
    assert len(dont_cache_b.layers) == 4
    assert dont_cache_b.layers[0].cached
    assert dont_cache_b.layers[1].cached
    assert not dont_cache_b.layers[2].cached
    assert not dont_cache_b.layers[3].cached

    builder = application.get_builder(dont_cache_b)
    builder.run(dont_cache_b.target_image, ["ls", "-1", "/asd"])


def test_stop_layering(target_image, application, build):
    """ utilize a playbook which halts caching """
    build.playbook_path = change_layering_playbook
    application.build(build)
    build = application.db.get_build(build.build_id)
    assert len(build.layers) == 3  # base image, first task and the final layer

    builder = application.get_builder(build)
    builder.run(build.target_image, ["ls", "-1", "/etc/passwd-lol"])


def test_file_caching_mechanism(tmpdir, target_image, application, build):
    """ make sure that we don't load from cache when a file was changed """
    t = str(tmpdir)
    pb_name = "file_caching.yaml"
    test_file_name = "a_bag_of_fun"
    file_caching_pb = os.path.join(data_dir, pb_name)
    p = os.path.join(t, pb_name)
    test_file = os.path.join(data_dir, test_file_name)
    f = os.path.join(t, test_file_name)

    shutil.copy(file_caching_pb, p)
    shutil.copy(test_file, f)

    with open(p) as fd:
        d = yaml.safe_load(fd)
        d[0]["tasks"][0]["copy"]["src"] = f
    with open(p, "w") as fd:
        yaml.safe_dump(d, fd)

    build.playbook_path = p
    second_build = Build.from_json(build.to_dict())
    cached_build = Build.from_json(build.to_dict())

    application.build(build)
    build = application.db.get_build(build.build_id)
    assert len(build.layers) == 2
    assert build.layers[0].cached
    assert not build.layers[1].cached

    # ideally this would be cached, but isn't now
    application.build(cached_build)
    cached_build = application.db.get_build(cached_build.build_id)
    assert len(cached_build.layers) == 2
    assert cached_build.layers[0].cached
    # since ansible doesn't track files and whether they changed, let's just make sure it works we expect it to work
    assert not cached_build.layers[1].cached

    # and now we test that if we change the file, it's not loaded from cache
    fun_content = "Much more fun, fun, fun!"
    with open(f, "w") as fd:
        fd.write(fun_content)

    application.build(second_build)
    second_build = application.db.get_build(second_build.build_id)
    assert not second_build.layers[1].cached

    builder = application.get_builder(second_build)
    out = builder.run(second_build.target_image, ["cat", "/fun"])
    assert out == fun_content



"""
Base class for builders
"""
import datetime
from enum import Enum

from ansible_bender.constants import TIMESTAMP_FORMAT


class ImageMetadata:
    """
    working_dir: str, path to a working directory within container image
    labels: dict with labels
    env_vars: dict with env vars
    cmd: str, command to run by default in the container
    user: str, username or uid; the container gets invoked with this user by default
    ports: list of str, ports to expose from container by default
    volumes: list of str; paths within the container which has data stored outside
             of the container
    """
    def __init__(self):
        self.working_dir = None
        self.labels = {}
        self.env_vars = {}
        self.cmd = None
        self.user = None
        self.ports = []
        self.volumes = []

    def to_dict(self):
        return {
            "working_dir": self.working_dir,
            "labels": self.labels,
            "env_vars": self.env_vars,
            "cmd": self.cmd,
            "user": self.user,
            "ports": self.ports,
            "volumes": self.volumes
        }

    @classmethod
    def from_json(cls, j):
        """ return Metadata instance from the provided json """
        m = cls()
        m.working_dir = j["working_dir"]
        m.labels = j["labels"]
        m.env_vars = j["env_vars"]
        m.cmd = j["cmd"]
        m.user = j["user"]
        m.ports = j["ports"]
        m.volumes = j["volumes"]
        return m


class Layer:
    """ This is an image layer """

    def __init__(self, content, layer_id, base_image_id, cached=None):
        """
        :param content: what's the content of the layer
        :param layer_id:
        :param base_image_id:
        :param cached: bool, was it loaded from cache?
        """
        self.content = content
        self.layer_id = layer_id
        self.base_image_id = base_image_id
        self.cached = cached

    def __str__(self):
        return f"layer_id={self.layer_id} cached={self.cached}"

    def __repr__(self):
        return f"Layer(layer_id={self.layer_id}, cached={self.cached}, base={self.base_image_id})"

    def to_dict(self):
        return {
            "content": self.content,
            "layer_id": self.layer_id,
            "base_image_id": self.base_image_id,
            "cached": self.cached
        }

    @classmethod
    def from_json(cls, j):
        return cls(
            j["content"],
            j["layer_id"],
            j["base_image_id"],
            cached=j["cached"]
        )


class Build:
    """ class which represents a build """
    def __init__(self):
        self.build_id = None  # PK, should be set by database
        self.playbook_path = None
        self.build_volumes = []  # volumes for the build container
        self.metadata = None  # Image metadata
        self.state = None  # enum, BuildState
        self.build_start_time = None
        self.build_finished_time = None
        self.base_image = None
        self.build_container = None
        self.target_image = None
        self.builder_name = None
        self.layers = []
        self.final_layer_id = None  # once the image is built, this is the final layer: content + metadata
        self.layer_index = {}  # this is an index for layers: `layer_id: Layer()`
        self.cache_tasks = True  # we cache by default, a user can opt out
        self.log_lines = []  # a list of strings
        self.layering = True
        self.debug = None
        self.verbose = None

    def to_dict(self):
        """ serialize """
        return {
            "build_id": self.build_id,
            "playbook_path": self.playbook_path,
            "build_volumes": self.build_volumes,
            "metadata": self.metadata.to_dict(),
            "state": self.state.value,
            "build_start_time": self.build_start_time.strftime(TIMESTAMP_FORMAT)
                if self.build_start_time else None,
            "build_finished_time": self.build_finished_time.strftime(TIMESTAMP_FORMAT)
                if self.build_finished_time else None,
            "base_image": self.base_image,
            "target_image": self.target_image,
            "builder_name": self.builder_name,
            "layers": [x.to_dict() for x in self.layers],
            "final_layer_id": self.final_layer_id,
            "layer_index": {x.layer_id: x.to_dict() for x in self.layers},
            "build_container": self.build_container,
            "cache_tasks": self.cache_tasks,
            # we could compress/base64 here, let's go for the easier solution first
            "log_lines": self.log_lines,
            "layering": self.layering,
            "debug": self.debug,
            "verbose": self.verbose
        }

    @classmethod
    def from_json(cls, j):
        """ return Build instance from the provided json """
        b = cls()
        b.build_id = j["build_id"]
        b.playbook_path = j["playbook_path"]
        b.build_volumes = j["build_volumes"]
        b.metadata = ImageMetadata.from_json(j["metadata"])
        b.state = BuildState(j["state"])
        b.build_start_time = None
        sta = j["build_start_time"]
        if sta:
            b.build_start_time = datetime.datetime.strptime(sta, TIMESTAMP_FORMAT)
        b.build_finished_time = None
        fin = j["build_finished_time"]
        if fin:
            b.build_finished_time = datetime.datetime.strptime(fin, TIMESTAMP_FORMAT)
        b.base_image = j["base_image"]
        b.target_image = j["target_image"]
        b.builder_name = j["builder_name"]
        b.layers = [Layer.from_json(x) for x in j["layers"]]
        b.final_layer_id = j.get("final_layer_id", None)
        b.layer_index = {layer_id: Layer.from_json(layer_data)
                         for layer_id, layer_data in j["layer_index"].items()}
        b.build_container = j["build_container"]
        b.cache_tasks = j["cache_tasks"]
        b.log_lines = j["log_lines"]
        b.layering = j["layering"]
        b.debug = j["debug"]
        b.verbose = j["verbose"]
        return b

    def record_layer(self, content, layer_id, base_image_id, cached=None):
        """
        record a new layer for this build

        :param content:
        :param layer_id:
        :param base_image_id:
        :param cached: bool, was it cached?
        """
        layer = Layer(content, layer_id, base_image_id, cached=cached)
        self.layers.append(layer)
        self.layer_index[layer_id] = layer

    def get_top_layer_id(self):
        """
        return id of the top layer, or None
        """
        if self.layers:
            return self.layers[-1].layer_id

    def get_target_image_id(self):
        """
        this is the preferred way of getting ID of the built image

        :return: str
        """
        if self.state not in [BuildState.DONE, BuildState.FAILED]:
            raise RuntimeError(f"Build have not finished yet, it is in state '{self.state.value}'.")
        return self.get_top_layer_id()

    def was_last_layer_cached(self):
        if self.layers:
            return self.layers[-1].cached

    def record_cache_entry(self, image_id):
        self.layer_index[image_id].cached = True

    def stop_layering(self):
        self.layering = False

    def is_layering_on(self):
        return self.layering


class BuildState(Enum):
    NEW = "new"
    IN_PROGRESS = "in_progress"
    DONE = "done"
    FAILED = "failed"


class Builder:
    ansible_connection = "default-value"
    name = "default-value"

    def __init__(self, build, debug=False):
        """
        :param build: instance of Build
        :param debug: bool, provide debug output if True
        """
        self.build = build
        self.ansible_host = None
        self.debug = debug
        self.python_interpr_prio = (
            "/usr/bin/python3",
            "/usr/local/bin/python3",
            "/usr/bin/python2",
            "/usr/local/bin/python2",
            "/usr/bin/python",
            "/usr/local/bin/python",
        )

    def create(self, build_volumes=None):
        """
        :param build_volumes: list of str, bind-mount specification: ["/host:/cont", ...]
        """

    def run(self, image_name, command):
        """
        run provided command in the selected image and return output

        :param image_name: str
        :param command: list of str
        :return: str (output)
        """

    def commit(self, image_name):
        """
        snapshot the artifact and create an image

        :param image_name: str, name the snapshot
        """

    def clean(self):
        """
        clean working container
        """

    def get_image_id(self, image_name):
        """ return image_id for provided image """

    def is_image_present(self, image_reference):
        """
        :return: True when the selected image is present, False otherwise
        """

    def is_base_image_present(self):
        """
        :return: True when the base image is present, False otherwise
        """
        return self.is_image_present(self.build.base_image)

    def pull(self):
        """
        pull base image
        """

    def push(self, build, target, force=False):
        """
        push built image into a remote location

        :param target: str, transport:details
        :param build: instance of Build
        :param force: bool, bypass checks if True
        :return: None
        """

    def find_python_interpreter(self):
        """
        find python executable in the base image, for prio order see constructor

        :return: str, path to python interpreter
        """

    def get_logs(self):
        """
        obtain logs for the selected build

        :return: list of str
        """



#!/usr/bin/python3

import os

from setuptools import find_packages, setup


def get_requirements():
    with open("./requirements.txt") as file:
        return file.readlines()

# https://packaging.python.org/guides/single-sourcing-package-version/
version = {}
with open("./ansible_bender/version.py") as fp:
    exec(fp.read(), version)

long_description = ''.join(open('README.md').readlines())


setup(
    name='ansible-bender',
    version=version["__version__"],
    description="A tool which builds container images using Ansible playbooks.",
    long_description=long_description,
    # long_description_content_type='text/markdown',
    packages=find_packages(exclude=['tests']),
    python_requires='>=3.6',
    install_requires=get_requirements(),
    entry_points='''
        [console_scripts]
        ab=ansible_bender.cli:main
    ''',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development',
        'Topic :: Utilities',
    ],
    keywords='containers,ansible,buildah',
    author='Tomas Tomecek',
    author_email='tomas@tomecek.net',
    url='https://github.com/TomasTomecek/ansible-bender',
    include_package_data=True,
)
